@@ -40,6 +40,33 @@ bool NavitiaBackend::needsLocationQuery(const Location &loc, AbstractBackend::Qu
     return !loc.hasCoordinate();
 }
 
+struct {
+    IndividualTransport::Mode mode;
+    IndividualTransport::Qualifier qualifier;
+    const char *name;
+} static constexpr const mode_table[] = {
+    { IndividualTransport::Walk, IndividualTransport::None, "walking" },
+    { IndividualTransport::Bike, IndividualTransport::None, "bike" },
+    { IndividualTransport::Bike, IndividualTransport::Park, "bike" }, // ### ?
+    { IndividualTransport::Bike, IndividualTransport::Rent, "bss" },
+    { IndividualTransport::Car, IndividualTransport::None, "car" },
+    { IndividualTransport::Car, IndividualTransport::Park, "car" },
+    { IndividualTransport::Car, IndividualTransport::Dropoff, "car_no_park" },
+    // TODO: ridesharing, taxi
+};
+
+static void addModeArgument(const QString &arg, const std::vector<IndividualTransport> &modes, QUrlQuery &query)
+{
+    for (const auto &mode : modes) {
+        const auto it = std::find_if(std::begin(mode_table), std::end(mode_table), [mode](const auto &entry) {
+            return entry.mode == mode.mode() && entry.qualifier == mode.qualifier();
+        });
+        if (it != std::end(mode_table)) {
+            query.addQueryItem(arg, QLatin1String((*it).name));
+        }
+    }
+}
+
 bool NavitiaBackend::queryJourney(const JourneyRequest &req, JourneyReply *reply, QNetworkAccessManager *nam) const
 {
     if ((req.modes() & JourneySection::PublicTransport) == 0) {
@@ -49,7 +76,7 @@ bool NavitiaBackend::queryJourney(const JourneyRequest &req, JourneyReply *reply
         return false;
     }
 
-    QUrl url = requestContext(req).value<QUrl>();
+    QUrl url = requestContextData(req).value<QUrl>();
     if (!url.isValid()) {
         url.setHost(m_endpoint);
         url.setPath(QStringLiteral("/v1") +
@@ -64,10 +91,13 @@ bool NavitiaBackend::queryJourney(const JourneyRequest &req, JourneyReply *reply
             query.addQueryItem(QStringLiteral("datetime_represents"), req.dateTimeMode() == JourneyRequest::Arrival ? QStringLiteral("arrival") : QStringLiteral("departure"));
         }
         query.addQueryItem(QStringLiteral("count"), QString::number(std::max(1, req.maximumResults())));
+        addModeArgument(QStringLiteral("first_section_mode[]"), req.accessModes(), query);
+        addModeArgument(QStringLiteral("last_section_mode[]"), req.egressModes(), query);
 
         // TODO: disable reply parts we don't care about
         query.addQueryItem(QStringLiteral("disable_geojson"), req.includePaths() ? QStringLiteral("false") : QStringLiteral("true"));
         query.addQueryItem(QStringLiteral("depth"), QStringLiteral("0")); // ### also has no effect?
+        query.addQueryItem(QStringLiteral("add_poi_infos[]"), QStringLiteral("bss_stands"));
         url.setQuery(query);
     }
     url.setScheme(QStringLiteral("https")); // force https in any case
@@ -168,7 +198,7 @@ bool NavitiaBackend::queryStopover(const StopoverRequest &req, StopoverReply *re
 
 bool NavitiaBackend::queryLocation(const LocationRequest &req, LocationReply *reply, QNetworkAccessManager *nam) const
 {
-    if ((req.types() & Location::Stop) == 0) {
+    if ((req.types() & (Location::Stop | Location::RentedVehicleStation)) == 0) {
         return false;
     }
 
@@ -179,8 +209,15 @@ bool NavitiaBackend::queryLocation(const LocationRequest &req, LocationReply *re
     QUrlQuery query;
     query.addQueryItem(QStringLiteral("disable_geojson"), QStringLiteral("true"));
     query.addQueryItem(QStringLiteral("depth"), QStringLiteral("1")); // 1 so administrative region elements are included
-    query.addQueryItem(QStringLiteral("type[]"), QStringLiteral("stop_area"));
     query.addQueryItem(QStringLiteral("count"), QString::number(std::max(1, req.maximumResults())));
+    if (req.types() & Location::Stop) {
+        query.addQueryItem(QStringLiteral("type[]"), QStringLiteral("stop_area"));
+    }
+    if (req.types() & Location::RentedVehicleStation) {
+        query.addQueryItem(QStringLiteral("type[]"), QStringLiteral("poi"));
+        query.addQueryItem(QStringLiteral("add_poi_infos[]"), QStringLiteral("bss_stands"));
+        query.addQueryItem(QStringLiteral("filter"), QStringLiteral("poi_type.id=poi_type:amenity:bicycle_rental"));
+    }
 
     if (req.hasCoordinate()) {
         url.setPath(
